% -- LSE_pulse_comb
function [coefficient, derivative] = ...
   LSE_pulse_comb
%%
% -----------------------------------------------
% Step Zero - Input parameters and Initialization
% -----------------------------------------------
Ts = 1e-7;
Tmin = -4e-9;                             
% Lower time interval limit
Tmax = 4e-9;                              
% Upper time interval limit
smp = 2^12;                               
% Number of samples
frequencysmoothingfactor = 4;             
% Frequency smoothing factor

dt = (Tmax - Tmin) / smp;
% sampling period
fs = 1/dt;
% sampling frequency
N = frequencysmoothingfactor * smp;
% number of samples (i.e. size of the FFT)
df = 1 / (N * dt);                        
% fundamental frequency

positivefrequency=linspace(0,(fs/2),N/2);
% Inizialization of the positive frequency axis
t=linspace(Tmin,Tmax,smp);                                      
% Inizialization of the time axis
alpha=get_alpha_value(1);                                
% Loading the alpha vector depending on the input 'i'


for i=1:15

% ---------------------------------------------
% Step One - Pulse waveforms in the time domain
% ---------------------------------------------

    derivative(i,:) = analytical_waveforms(t,i,alpha(i));
    derivative(i,:) = (derivative(i,:)/ max(abs(derivative(i,:))));
    
end

% ---------------------------------------------
% Step Two - Determination of the LSE solution to the
% approximation problem
% ---------------------------------------------

timeemissionmask = time_mask(Tmin,Tmax,smp);   
% Application of the LSE method
coefficient = sqrt(Ts * 377) *   lsqlin(derivative',timeemissionmask');
X = fft(coefficient'*derivative,N);
% double-sided MATLAB amplitude spectrum
X = X / N;
% conversion from MATLAB spectrum to fourier spectrum
E = fftshift(abs(X).^2/(df^2));
% DOUBLE-SIDED ESD
Ess = 2 * E((N/2+1):N);
% SINGLE-SIDED ESD  
PSD = 10 * log10 ((1/Ts) * Ess / 377) + 90;
% PSD of the combination in dBm/MHz

% -----------------------------
% Step Three - Graphical output
% -----------------------------

emissionmask = generate_mask(N, fs);
% Loading the emission mask on N/2 points

figure();
plot(positivefrequency/1e9,...
   emissionmask,'LineWidth',4,'Color',[0.33 0.66 1]); 
hold on;
PF = plot(positivefrequency/1e9, PSD);
PFF = plot(positivefrequency/1e9, PSD-max(PSD-emissionmask),'--');
set(PF,'LineWidth',[2]);
AX=gca;
T=title('LSE combination');
X=xlabel('Frequency (GHz)');
Y=ylabel('PSD  (dBm/MHz)');
grid on;
axis([0 10 -200 0]);
alphavalue = '\alpha = 0.714 ns';
%text(8, -100, alphavalue,'BackgroundColor', [1 1 1]);
text(6, -150, 'LSE combination', 'BackgroundColor',...
   [1 1 1]);
text(5, -30, 'FCC UWB indoor emission mask',...
   'BackgroundColor', [1 1 1]);